#include<iostream>
#include<cmath>
using namespace std;

const int MAXN=1e6, MAXM=(1<<(int)ceil(log2(MAXN)));

long long nums[2*MAXM][6][6], cnt[2*MAXM], lazy[2*MAXM], s[2*MAXM], pow10[6], a[MAXN];

void Erase(int idx, int t)
{
    for(int i=1; i<6; i++)
    {
        for(int j=0; j<i; j++)
        {
            if(t!=0)
            {
                nums[idx][i][j]-=nums[2*idx][i][(j+cnt[2*idx])%i];
            }

            if(t!=1)
            {
                nums[idx][i][j]-=nums[2*idx+1][i][(j+cnt[2*idx+1])%i];
            }
        }
    }
}

void PrepSum(int idx, int t)
{
    for(int i=1; i<6; i++)
    {
        for(int j=0; j<i; j++)
        {
            if(t!=0)
            {
                nums[idx][i][j]+=nums[2*idx][i][(j+cnt[2*idx])%i];
            }

            if(t!=1)
            {
                nums[idx][i][j]+=nums[2*idx+1][i][(j+cnt[2*idx+1])%i];
            }
        }
    }
}

void Update(int idx, int l, int r)
{
    if(s[idx]==1)
    {
        cnt[idx]++;

        return;
    }

    if(lazy[idx]!=0)
    {
        cnt[2*idx]+=lazy[idx];
        cnt[2*idx+1]+=lazy[idx];
        lazy[2*idx]+=lazy[idx];
        lazy[2*idx+1]+=lazy[idx];
        lazy[idx]=0;
    }

    if(l==0 && r==s[idx]-1)
    {
        cnt[idx]++;
        lazy[idx]++;

        return;
    }

    if(l>=s[2*idx])
    {
        Erase(idx, 0);
        Update(2*idx+1, l-s[2*idx], r-s[2*idx]);
        PrepSum(idx, 0);

        return;
    }

    if(r<s[2*idx])
    {
        Erase(idx, 1);
        Update(2*idx, l, r);
        PrepSum(idx, 1);

        return;
    }

    Erase(idx, 2);
    Update(2*idx, l, s[2*idx]-1);
    Update(2*idx+1, 0, r-s[2*idx]);
    PrepSum(idx, 2);

    return;
}

long long CalcSum(int idx)
{
    //cout << idx << " - " << cnt[idx] << " ";
    long long res=0;
    for(int i=1; i<6; i++)
    {
        for(int j=0; j<i; j++)
        {
            res+=pow10[j]*nums[idx][i][(j+cnt[idx])%i];
        }
    }

    //cout << res << "\n";

    return res;
}

long long GetSum(int idx, int l, int r)
{
    if(s[idx]==1)
    {
        return CalcSum(idx);
    }

    if(lazy[idx]!=0)
    {
        cnt[2*idx]+=lazy[idx];
        cnt[2*idx+1]+=lazy[idx];
        lazy[2*idx]+=lazy[idx];
        lazy[2*idx+1]+=lazy[idx];
        lazy[idx]=0;
    }

    if(l==0 && r==s[idx]-1)
    {
        return CalcSum(idx);
    }

    long long res;
    if(l>=s[2*idx])
    {
        res=GetSum(2*idx+1, l-s[2*idx], r-s[2*idx]);
    }
    else if(r<s[2*idx])
    {
        res=GetSum(2*idx, l, r);
    }
    else
    {
        res=GetSum(2*idx, l, s[2*idx]-1)+GetSum(2*idx+1, 0, r-s[2*idx]);
    }

    return res;
}

void BuildSeg(int n)
{
    int m=(1<<(int)ceil(log2(n)));
    for(int i=m; i<m+n; i++)
    {
        cnt[i]=0, lazy[i]=0, s[i]=1;

        for(int j=1; j<6; j++)
        {
            for(int u=0; u<j; u++)
            {
                nums[i][j][u]=0;
            }
        }

        int d=log10(a[i-m])+1, u=0;
        while(a[i-m]>0)
        {
            nums[i][d][u]=a[i-m]%10;
            a[i-m]/=10;
            u++;
        }
    }

    for(int i=m+n; i<2*m; i++)
    {
        cnt[i]=0, lazy[i]=0, s[i]=0;

        for(int j=1; j<6; j++)
        {
            for(int u=0; u<j; u++)
            {
                nums[i][j][u]=0;
            }
        }
    }

    for(int i=m-1; i>0; i--)
    {
        cnt[i]=0, lazy[i]=0, s[i]=s[2*i]+s[2*i+1];

        PrepSum(i, 2);
    }

    return;
}

int main()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);

    pow10[0]=1;
    for(int i=1; i<6; i++)
    {
        pow10[i]=pow10[i-1]*10;
    }

    int n, q;
    cin >> n >> q;

    for(int i=0; i<n; i++)
    {
        cin >> a[i];
    }

    BuildSeg(n);

    while(q--)
    {
        int t, l, r;
        cin >> t >> l >> r;
        l--, r--;

        if(t==1)
        {
            Update(1, l, r);
        }
        else
        {
            cout << GetSum(1, l, r) << "\n";
        }
    }

    return 0;
}
